// TCP socket server demo

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

public class SocketServer
extends Frame
implements ActionListener, ReceiverWindow
{
  Button btnListen;
  Button btnSend;
  Button btnDisco;
  Checkbox checkAppend;
  TextField editHost;
  TextField editPort;
  TextField editSend;
  TextField editStatus;
  TextArea area;
  Socket sockConn = null;
  ServerSocket sockListen = null;
  OutputStreamWriter osw;
  InputStreamReader isr;

  public static void main(String[] args)
  {
    SocketServer wnd = new SocketServer();
    wnd.addWindowListener(
      new WindowAdapter() {
        public void windowClosing( WindowEvent e ) {
          System.exit(0);
        }
      });
    wnd.setVisible(true);
  }


  // Callback functions for our helper threads:
  public void textReceived( String s )
  {
    if( s == null )
      disconnect( "Connection closed by remote host" );
    else
      area.append( s );
  }

  public void connAccepted( Socket sock )
  {
    sockConn = sock;
    try {
      sockConn.setSoTimeout( 100 );  // receive timeout
      osw = new OutputStreamWriter( sockConn.getOutputStream() );
      isr = new InputStreamReader( sockConn.getInputStream() );
      ReceiverThread th = new ReceiverThread( this, isr );
      th.start();
    } catch( IOException e ) {
      disconnect( ""+e );
      return;
    }
    editStatus.setText( "Connection from "
        + sockConn.getInetAddress().getHostAddress() + ", port " + sockConn.getPort() );
    area.setText("");
    btnSend.setEnabled( true );
  }

  public SocketServer()
  {
    super("TCP Simple Server");
    setBackground( SystemColor.control );
    String myIP;
    try {
      myIP = InetAddress.getLocalHost().getHostAddress();
    } catch( UnknownHostException e ) {
      myIP = "???";
    }
    // Set layout and start adding components
    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc;
    setLayout(gbl);

    // first row: two labels
    Label label;
    gbc = makegbc( 0, 0 );
    label = new Label( "Local Host" );
    gbl.setConstraints( label, gbc );
    add( label );
    gbc = makegbc( 1, 0 );
    label = new Label( "Local Port" );
    gbl.setConstraints( label, gbc );
    add( label );

    // second row: two text fields and a button
    gbc = makegbc( 0, 1 );
    gbc.weightx = 4;
    editHost = new TextField( myIP );
    editHost.setEditable( false );
    gbl.setConstraints( editHost, gbc );
    add( editHost );
    gbc = makegbc( 1, 1 );
    editPort = new TextField("8000");
    gbl.setConstraints( editPort, gbc );
    add( editPort );
    btnListen = new Button("Listen");
    btnListen.addActionListener( this );
    gbc = makegbc( 2, 1 );
    gbl.setConstraints( btnListen, gbc);
    add( btnListen );

    // third row: label and checkbox
    gbc = makegbc( 0, 2 );
    label = new Label( "Send Data" );
    gbl.setConstraints( label, gbc );
    add( label );
    gbc = makegbc( 1, 2 );
    checkAppend = new Checkbox ( "Append CR/LF" );
    checkAppend.setState( true );
    gbl.setConstraints( checkAppend, gbc );
    add( checkAppend );

    // fourth row: (wide) text field and button
    gbc = makegbc( 0, 3 );
    gbc.gridwidth = 2;
    editSend = new TextField("");
    gbl.setConstraints( editSend, gbc );
    add( editSend );
    btnSend = new Button("Send");
    btnSend.setEnabled( false );
    btnSend.addActionListener( this );
    gbc = makegbc( 2, 3 );
    gbl.setConstraints( btnSend, gbc );
    add( btnSend );

    // fifth and sixth row: label and text area
    gbc = makegbc( 0, 4 );
    label = new Label( "Receive Data" );
    gbl.setConstraints( label, gbc );
    add( label );
    area = new TextArea( 5, 40 );
    area.setEditable( false );
    gbc = makegbc(0, 5 );
    gbc.gridwidth = 2;
    gbc.weighty = 1;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(area, gbc);
    add( area );

    // seventh and eight row: label, (wide) text field and button
    gbc = makegbc( 0, 6 );
    label = new Label( "Status" );
    gbl.setConstraints( label, gbc );
    add( label );
    gbc = makegbc( 0, 7 );
    gbc.gridwidth = 2;
    editStatus = new TextField("Ready");
    editStatus.setEditable( false );
    gbl.setConstraints( editStatus, gbc );
    add( editStatus );
    btnDisco = new Button("Disconnect");
    btnDisco.setEnabled( false );
    btnDisco.addActionListener( this );
    gbc = makegbc( 2, 7 );
    gbl.setConstraints( btnDisco, gbc );
    add( btnDisco );

    // finalize layout:
    pack();
  }

  private GridBagConstraints makegbc( int x, int y )
  {
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.weightx = 1;
    gbc.insets = new Insets( 2, 2, 2, 2 );
    return gbc;
  }

  private void disconnect( String msg )
  {
    editStatus.setText( msg );
    if( sockConn != null )
      try { sockConn.close(); } catch( IOException e ) {}
    if( sockListen != null )
      try { sockListen.close(); } catch( IOException e ) {}
    sockConn = null;
    sockListen = null;
    btnListen.setEnabled( true );
    btnSend.setEnabled( false );
    btnDisco.setEnabled( false );
  }

  public void actionPerformed(ActionEvent event)
  {
    String cmd = event.getActionCommand();
    if( cmd.equals( "Listen" ) ) {
      btnListen.setEnabled( false );
      editStatus.setText( "Listening..." );
      String s = editPort.getText();
      int port = Integer.parseInt( s );
      try {
        sockListen = new ServerSocket( port );
        ListenThread th = new ListenThread( this, sockListen );
        th.start();
      } catch( IOException e ) {
        disconnect( ""+e );
        return;
      }
      btnDisco.setEnabled( true );
    } else if( cmd.equals( "Send" ) ) {
      String s = editSend.getText();
      try {
        osw.write( s );
        if( checkAppend.getState() )
          osw.write( "\r\n" );
        osw.flush();
        editSend.setText("");
      } catch( IOException e ) {
        disconnect( ""+e );
      }
    } else if( cmd.equals( "Disconnect" ) ) {
      disconnect( "Ready" );
    }
  }

}



class ListenThread
extends Thread
{
  // This thread calls accept() for the given ServerSocket and runs
  // until it returns. If succesful, the resulting Socket is reported
  // back to the calling window.
  SocketServer ss;
  ServerSocket sockListen;

  public ListenThread( SocketServer ss, ServerSocket sock )
  {
    super();
    this.ss = ss;
    sockListen = sock;
  }

  public void run()
  {
    Socket sockAccepted;
    try {
      sockAccepted = sockListen.accept();
    } catch (IOException e) {
      System.err.println("ListenThread: " + e.toString());
      return;
    }
    ss.connAccepted( sockAccepted );
  }
}

