// UDP communication demo

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

public class UdpExchange
extends Frame
implements ActionListener, TextListener
{
  Button btnBind;
  Button btnSend;
  Checkbox checkAppend;
  TextField editLHost, editLPort;
  TextField editRHost, editRPort;
  TextField editSend;
  TextField editStatus;
  TextArea area;
  DatagramSocket sock = null;

  public static void main(String[] args)
  {
    UdpExchange wnd = new UdpExchange();
    wnd.addWindowListener(
      new WindowAdapter() {
        public void windowClosing( WindowEvent e ) {
          System.exit(0);
        }
      });
    wnd.setVisible(true);
  }

  // Callback function for our helper thread
  public void packetReceived( DatagramPacket pkt )
  {
    editStatus.setText( "Datagram from " + pkt.getAddress().getHostAddress()
      + ", port " + pkt.getPort() + ", " + pkt.getLength() + " byte" );
    String s = new String( pkt.getData(), 0, pkt.getLength() );
    area.append( s );
  }

  boolean allocSocket( boolean force )
  {
    if( sock != null ) {
      if( force )
        sock.close();
      else
        return true;
    }
    int port = Integer.parseInt( editLPort.getText() );
    sock = null;
    try {
      sock = new DatagramSocket( port );
      UdpReceiverThread th = new UdpReceiverThread( this, sock );
      th.start();
    } catch( IOException e ) {
      if( sock != null )
        sock.close();
      sock = null;
      return false;
    }
    editLPort.setText( ""+sock.getLocalPort() );
    return true;
  }

  public UdpExchange()
  {
    super("UDP Simple Data Exchange");
    setBackground( SystemColor.control );
    String myIP;
    try {
      myIP = InetAddress.getLocalHost().getHostAddress();
    } catch( UnknownHostException e ) {
      myIP = "???";
    }
    // Set layout and start adding components
    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc;
    setLayout(gbl);

    // first row: two labels
    Label label;
    gbc = makegbc( 0, 0 );
    label = new Label( "Local Host" );
    gbl.setConstraints( label, gbc );
    add( label );
    gbc = makegbc( 1, 0 );
    label = new Label( "Local Port" );
    gbl.setConstraints( label, gbc );
    add( label );

    // second row: two text fields and a button
    gbc = makegbc( 0, 1 );
    gbc.weightx = 4;
    editLHost = new TextField( myIP );
    editLHost.setEditable( false );
    gbl.setConstraints( editLHost, gbc );
    add( editLHost );
    gbc = makegbc( 1, 1 );
    editLPort = new TextField( "0" );
    editLPort.addTextListener( this );
    gbl.setConstraints( editLPort, gbc );
    add( editLPort );
    btnBind = new Button("Bind");
    btnBind.setEnabled( false );
    btnBind.addActionListener( this );
    gbc = makegbc( 2, 1 );
    gbl.setConstraints( btnBind, gbc);
    add( btnBind );

    // third row: two labels
    gbc = makegbc( 0, 2 );
    label = new Label( "Remote Host" );
    gbl.setConstraints( label, gbc );
    add( label );
    gbc = makegbc( 1, 2 );
    label = new Label( "Remote Port" );
    gbl.setConstraints( label, gbc );
    add( label );

    // fourth row: two text fields and a button
    gbc = makegbc( 0, 3 );
    gbc.weightx = 4;
    editRHost = new TextField();
    gbl.setConstraints( editRHost, gbc );
    add( editRHost );
    gbc = makegbc( 1, 3 );
    editRPort = new TextField( "8500" );
    gbl.setConstraints( editRPort, gbc );
    add( editRPort );

    // fifth row: label and checkbox
    gbc = makegbc( 0, 4 );
    label = new Label( "Send Data" );
    gbl.setConstraints( label, gbc );
    add( label );
    gbc = makegbc( 1, 4 );
    checkAppend = new Checkbox ( "Append CR/LF" );
    gbl.setConstraints( checkAppend, gbc );
    add( checkAppend );

    // sixth row: (wide) text field and button
    gbc = makegbc( 0, 5 );
    gbc.gridwidth = 2;
    editSend = new TextField("");
    gbl.setConstraints( editSend, gbc );
    add( editSend );
    btnSend = new Button("Send");
    btnSend.addActionListener( this );
    gbc = makegbc( 2, 5 );
    gbl.setConstraints( btnSend, gbc );
    add( btnSend );

    // seventh and eight row: label and text area
    gbc = makegbc( 0, 6 );
    label = new Label( "Receive Data" );
    gbl.setConstraints( label, gbc );
    add( label );
    area = new TextArea( 5, 40 );
    area.setEditable( false );
    gbc = makegbc(0, 7 );
    gbc.gridwidth = 2;
    gbc.weighty = 1;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(area, gbc);
    add( area );

    // ninth and tenth row: label, (wide) text field
    gbc = makegbc( 0, 8 );
    label = new Label( "Status" );
    gbl.setConstraints( label, gbc );
    add( label );
    gbc = makegbc( 0, 9 );
    gbc.gridwidth = 2;
    editStatus = new TextField("Ready");
    editStatus.setEditable( false );
    gbl.setConstraints( editStatus, gbc );
    add( editStatus );

    // finalize layout:
    pack();
  }

  private GridBagConstraints makegbc( int x, int y )
  {
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.weightx = 1;
    gbc.insets = new Insets( 2, 2, 2, 2 );
    return gbc;
  }

  public void textValueChanged( TextEvent e )
    {
    // Enable the "Bind" button, when the user edits the local port.
    if( editLPort.hasFocus() )
      btnBind.setEnabled( true );
    }
  public void actionPerformed(ActionEvent event)
  {
    String cmd = event.getActionCommand();
    if( cmd.equals( "Bind" ) ) {
      btnBind.setEnabled( false );
      area.setText( "" );
      allocSocket( true );
    } else if( cmd.equals( "Send" ) ) {
      allocSocket( false );
      String s = editSend.getText();
      if( checkAppend.getState() )
        s += "\r\n";
      String host = editRHost.getText();
      int port = Integer.parseInt( editRPort.getText() );
      try {
        DatagramPacket pkt = new DatagramPacket( s.getBytes(), s.length(),
          InetAddress.getByName( host ), port );
        sock.send( pkt );
        editSend.setText("");
      } catch( IOException e ) {
        editStatus.setText( ""+e );
      }
    }
  }

}


class UdpReceiverThread
extends Thread
{
  // This thread receives incoming data from a socket and reports it back
  // to the calling window. It runs until the socket is closed.
  UdpExchange ux;
  DatagramSocket sock;

  public UdpReceiverThread( UdpExchange ux, DatagramSocket sock )
  {
    this.ux = ux;
    this.sock = sock;
  }

  public void run()
  {
    byte b[] = new byte[ 100 ];
    DatagramPacket pkt = new DatagramPacket( b, b.length );
    for( ;; ) {
      try {
        sock.receive( pkt );
        ux.packetReceived( pkt );
      } catch (SocketTimeoutException e) {
        // timeout, try again
      } catch (IOException e) {
        //System.err.println("ReceiverThread: " + e.toString());
        break;
      }
    }
  }
}
